@@ -2,7 +2,7 @@
  * Copyright 2005-2007 LAMP/EPFL
  * @author  Martin Odersky
  */
-// $Id: Interpreter.scala 11066 2007-05-16 12:58:54Z michelou $
+// $Id: Interpreter.scala 11878 2007-06-01 13:15:03Z spoon $
 
 package scala.tools.nsc
 
@@ -54,7 +54,7 @@ import nsc.{InterpreterResults=>IR}
  *    properly, because rebinding at the Java level is technically difficult.
  *  </p>
  */
-class Interpreter(val settings: Settings, reporter: Reporter, out: PrintWriter) {
+class Interpreter(val settings: Settings, out: PrintWriter) {
   import symtab.Names
   import compiler.Traverser
   import compiler.{Tree, TermTree,
@@ -67,14 +67,8 @@ class Interpreter(val settings: Settings, reporter: Reporter, out: PrintWriter)
 
   /** construct an interpreter that reports to Console */
   def this(settings: Settings) =
-    this(settings,
-         new ConsoleReporter(settings),
-         new PrintWriter(new ConsoleWriter, true))
+    this(settings, new PrintWriter(new ConsoleWriter, true))
 
-  /** construct an interpreter that uses the specified in and out streams */
-  def this(settings: Settings, out: PrintWriter) =
-    this(settings, new ConsoleReporter(settings, null, out), out)
-  
   /** whether to print out result lines */
   private var printResults: Boolean = true
 
@@ -98,12 +92,16 @@ class Interpreter(val settings: Settings, reporter: Reporter, out: PrintWriter)
 
   /** directory to save .class files to */
   val classfilePath = File.createTempFile("scalaint", "")
-  classfilePath.delete  // the file is created as a file; make it a directory
-  classfilePath.mkdirs
+  classfilePath.delete()  // the file is created as a file; make it a directory
+  classfilePath.mkdirs()
 
   /* set up the compiler's output directory */
   settings.outdir.value = classfilePath.getPath
 
+  object reporter extends ConsoleReporter(settings, null, out) {
+    override def printMessage(msg: String) = out.println(clean(msg))
+  }
+
   /** the compiler to compile expressions with */
   val compiler: Global = new Global(settings, reporter)
 
@@ -187,6 +185,11 @@ class Interpreter(val settings: Settings, reporter: Reporter, out: PrintWriter)
     return str.substring(0, maxpr)
   }
 
+  /** Clean up a string for output */
+  private def clean(str: String) =
+    truncPrintString(Interpreter.stripWrapperGunk(str))
+
+
   /** Compute imports that allow definitions from previous
    *  requests to be visible in a new request.  Returns
    *  three pieces of related code:
@@ -414,9 +417,6 @@ class Interpreter(val settings: Settings, reporter: Reporter, out: PrintWriter)
    *  @return     ...
    */
   def interpret(line: String): IR.Result = {
-    def clean(str: String) =
-      truncPrintString(Interpreter.stripWrapperGunk(str))
-
     // parse
     val trees = parse(line) match {
       case None => return IR.Incomplete
@@ -451,7 +451,7 @@ class Interpreter(val settings: Settings, reporter: Reporter, out: PrintWriter)
     if (succeeded) IR.Success else IR.Error
   }
 
-  /** A counter used for numbering objects created by bind() */
+  /** A counter used for numbering objects created by <code>bind()</code>. */
   private var binderNum = 0
 
   /** Bind a specified name to a specified value.  The name may
@@ -464,7 +464,7 @@ class Interpreter(val settings: Settings, reporter: Reporter, out: PrintWriter)
    */
   def bind(name: String, boundType: String, value: Any): IR.Result = {
     val binderName = "binder" + binderNum
-    binderNum = binderNum + 1
+    binderNum += 1
 
     compileString(
         "object " + binderName +
@@ -497,10 +497,14 @@ class Interpreter(val settings: Settings, reporter: Reporter, out: PrintWriter)
    *    class files for this instance.  This cannot safely be done after
    *    each command is executed because of Java's demand loading.
    *  </p>
+   *  <p>
+   *    Also, this flushes the reporter's output.
+   *  </p>
    */
-  def close(): Unit =
+  def close() {
     Interpreter.deleteRecursively(classfilePath)
-
+    reporter.flush()
+  }
 
   /** A traverser that finds all mentioned identifiers, i.e. things
       that need to be imported.
@@ -878,8 +882,7 @@ object Interpreter {
    *  from an interpreter output string.
    */
   def stripWrapperGunk(str: String): String = {
-    val wrapregex = "line[0-9]+\\$object(\\$\\$iw)*"
-    str.replaceAll(wrapregex+"\\.", "")
-       .replaceAll(wrapregex+"\\$", "")
+    val wrapregex = "(line[0-9]+\\$object[$.])?(\\$iw[$.])*"
+    str.replaceAll(wrapregex, "")
   }
 }
